@@ -25,7 +25,8 @@
 
 import codecs
 
-from gnomemusic.player import PlaybackStatus, RepeatType
+from gnomemusic.gstplayer import Playback
+from gnomemusic.player import RepeatType
 from gnomemusic.grilo import grilo
 from gnomemusic.playlists import Playlists
 from gnomemusic.utils import View
@@ -221,14 +222,17 @@ class MediaPlayer2Service(Server):
 
         self.app = app
         self.player = app.get_active_window().player
-        self.player.connect('current-changed', self._on_current_changed)
-        self.player.connect('thumbnail-updated', self._on_thumbnail_updated)
+        self.player.connect(
+            'song-changed', self._on_current_song_changed)
         self.player.connect('playback-status-changed', self._on_playback_status_changed)
         self.player.connect('repeat-mode-changed', self._on_repeat_mode_changed)
         self.player.connect('volume-changed', self._on_volume_changed)
         self.player.connect('prev-next-invalidated', self._on_prev_next_invalidated)
         self.player.connect('seeked', self._on_seeked)
         self.player.connect('playlist-changed', self._on_playlist_changed)
+        self.player_toolbar = app.get_active_window().player_toolbar
+        self.player_toolbar.connect(
+            'thumbnail-updated', self._on_thumbnail_updated)
         playlists = Playlists.get_default()
         playlists.connect('playlist-created', self._on_playlists_count_changed)
         playlists.connect('playlist-deleted', self._on_playlists_count_changed)
@@ -242,9 +246,9 @@ class MediaPlayer2Service(Server):
     @log
     def _get_playback_status(self):
         state = self.player.get_playback_status()
-        if state == PlaybackStatus.PLAYING:
+        if state == Playback.PLAYING:
             return 'Playing'
-        elif state == PlaybackStatus.PAUSED:
+        elif state == Playback.PAUSED:
             return 'Paused'
         else:
             return 'Stopped'
@@ -396,15 +400,15 @@ class MediaPlayer2Service(Server):
 
     @log
     def _get_active_playlist(self):
-        playlist = self._get_playlist_from_id(self.player.playlistId) \
-            if self.player.playlistType == 'Playlist' else None
+        playlist = self._get_playlist_from_id(self.player.playlist_id) \
+            if self.player.playlist_type == 'Playlist' else None
         playlistName = utils.get_media_title(playlist) \
             if playlist else ''
         return (playlist is not None,
                 (self._get_playlist_path(playlist), playlistName, ''))
 
     @log
-    def _on_current_changed(self, player, data=None):
+    def _on_current_song_changed(self, player, current_iter, data=None):
         if self.player.repeat == RepeatType.SONG:
             self.Seeked(0)
 
@@ -494,11 +498,11 @@ class MediaPlayer2Service(Server):
 
     @log
     def _on_playlist_modified(self, path=None, _iter=None, data=None):
-        if self.player.currentTrack and self.player.currentTrack.valid():
-            path = self.player.currentTrack.get_path()
-            currentTrack = self.player.playlist[path][self.player.Field.SONG]
+        if self.player.current_song and self.player.current_song.valid():
+            path = self.player.current_song.get_path()
+            current_song = self.player.playlist[path][self.player.Field.SONG]
             track_list = self._get_track_list()
-            self.TrackListReplaced(track_list, self._get_media_id(currentTrack))
+            self.TrackListReplaced(track_list, self._get_media_id(current_song))
             self.PropertiesChanged(MediaPlayer2Service.MEDIA_PLAYER2_TRACKLIST_IFACE,
                                    {
                                        'Tracks': GLib.Variant('ao', track_list),
@@ -532,10 +536,10 @@ class MediaPlayer2Service(Server):
         self.app.quit()
 
     def Next(self):
-        self.player.play_next()
+        self.player.next()
 
     def Previous(self):
-        self.player.play_previous()
+        self.player.previous()
 
     def Pause(self):
         self.player.set_playing(False)
@@ -544,7 +548,7 @@ class MediaPlayer2Service(Server):
         self.player.play_pause()
 
     def Stop(self):
-        self.player.Stop()
+        self.player.stop()
 
     def Play(self):
         if self.player.playlist is not None:
@@ -594,19 +598,18 @@ class MediaPlayer2Service(Server):
             media = track[self.player.Field.SONG]
             if track_id == self._get_media_id(media):
                 self.player.set_playlist(
-                    self.player.playlistType, self.player.playlistId,
+                    self.player.playlist_type, self.player.playlist_id,
                     self.player.playlist, track.iter)
-
                 self.player.play()
                 return
 
-    def TrackListReplaced(self, tracks, current_track):
+    def TrackListReplaced(self, tracks, current_song):
         self.con.emit_signal(None,
                              '/org/mpris/MediaPlayer2',
                              MediaPlayer2Service.MEDIA_PLAYER2_TRACKLIST_IFACE,
                              'TrackListReplaced',
                              GLib.Variant.new_tuple(GLib.Variant('ao', tracks),
-                                                    GLib.Variant('o', current_track)))
+                                                    GLib.Variant('o', current_song)))
 
     def TrackAdded(self, metadata, after_track):
         self.con.emit_signal(None,
@@ -687,8 +690,8 @@ class MediaPlayer2Service(Server):
                 'MaximumRate': GLib.Variant('d', 1.0),
                 'CanGoNext': GLib.Variant('b', self.player.has_next()),
                 'CanGoPrevious': GLib.Variant('b', self.player.has_previous()),
-                'CanPlay': GLib.Variant('b', self.player.currentTrack is not None),
-                'CanPause': GLib.Variant('b', self.player.currentTrack is not None),
+                'CanPlay': GLib.Variant('b', self.player.current_song is not None),
+                'CanPause': GLib.Variant('b', self.player.current_song is not None),
                 'CanSeek': GLib.Variant('b', True),
                 'CanControl': GLib.Variant('b', True),
             }
