@@ -30,8 +30,8 @@ from gi.repository import GLib, GObject, Gtk, Gdk
 from gnomemusic import log
 from gnomemusic.albumartcache import Art
 from gnomemusic.grilo import grilo
-from gnomemusic.toolbar import ToolbarState
 from gnomemusic.views.baseview import BaseView
+from gnomemusic.widgets.headerbar import HeaderBar
 from gnomemusic.widgets.albumwidget import AlbumWidget
 from gnomemusic.widgets.coverstack import CoverStack
 import gnomemusic.utils as utils
@@ -61,7 +61,6 @@ class AlbumsView(BaseView):
     def _on_changes_pending(self, data=None):
         if (self._init and not self._header_bar.selection_mode):
             self._offset = 0
-            self._init = True
             GLib.idle_add(self.populate)
             grilo.changes_pending['Albums'] = False
 
@@ -90,40 +89,27 @@ class AlbumsView(BaseView):
 
     @log
     def _back_button_clicked(self, widget, data=None):
-        self._header_bar.reset_header_title()
+        self._header_bar.state = HeaderBar.State.MAIN
         self.set_visible_child(self._grid)
 
     @log
     def _on_child_activated(self, widget, child, user_data=None):
         item = child.media_item
 
-        if self._star_handler.star_renderer_click:
-            self._star_handler.star_renderer_click = False
-            return
-
         # Toggle the selection when in selection mode
         if self.selection_mode:
             child.check.set_active(not child.check.get_active())
             return
 
-        title = utils.get_media_title(item)
-        self._escaped_title = title
-        self._artist = utils.get_artist_name(item)
-
+        # Update and display the album widget if not in selection mode
         self._album_widget.update(
-            self._artist, title, item, self._header_bar,
-            self._selection_toolbar)
+            item, self._header_bar, self._selection_toolbar)
 
-        self._header_bar.set_state(ToolbarState.CHILD_VIEW)
-        self._header_bar.header_bar.set_title(self._escaped_title)
-        self._header_bar.header_bar.sub_title = self._artist
+        self._header_bar.props.state = HeaderBar.State.CHILD
+        self._header_bar.props.title = utils.get_album_title(item)
+        self._header_bar.props.subtitle = utils.get_artist_name(item)
         self.set_visible_child(self._album_widget)
 
-    @log
-    def update_title(self):
-        self._header_bar.header_bar.set_title(self._escaped_title)
-        self._header_bar.header_bar.sub_title = self._artist
-
     @log
     def populate(self):
         self._window.notifications_popup.push_loading()
@@ -133,7 +119,7 @@ class AlbumsView(BaseView):
     def get_selected_songs(self, callback):
         # FIXME: we call into private objects with full knowledge of
         # what is there
-        if self._header_bar._state == ToolbarState.CHILD_VIEW:
+        if self._header_bar.props.state == HeaderBar.State.CHILD:
             callback(self._album_widget._disc_listbox.get_selected_items())
         else:
             self.items_selected = []
@@ -151,6 +137,7 @@ class AlbumsView(BaseView):
             # Add to the flowbox
             child = self._create_album_item(item)
             self._view.add(child)
+            self._offset += 1
         elif remaining == 0:
             self._view.show()
 
@@ -166,6 +153,7 @@ class AlbumsView(BaseView):
                     priority=GLib.PRIORITY_LOW)
 
             self._window.notifications_popup.pop_loading()
+            self._init = False
 
     def _create_album_item(self, item):
         artist = utils.get_artist_name(item)
