@@ -1,4 +1,4 @@
-# Copyright (c) 2016 The GNOME Music Developers
+# Copyright 2018 The GNOME Music Developers
 #
 # GNOME Music is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,13 +22,13 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-from gettext import gettext as _, ngettext
+import gi
+gi.require_version("Gd", "1.0")
 from gi.repository import Gd, GdkPixbuf, GObject, Gtk
 
 from gnomemusic import log
 from gnomemusic.grilo import grilo
 from gnomemusic.widgets.starhandlerwidget import StarHandlerWidget
-import gnomemusic.utils as utils
 
 
 class BaseView(Gtk.Stack):
@@ -92,7 +92,7 @@ class BaseView(Gtk.Stack):
 
         self._star_handler = StarHandlerWidget(self, 9)
         self._window = window
-        self._header_bar = window.toolbar
+        self._header_bar = window.headerbar
         self._selection_toolbar = window.selection_toolbar
         self._header_bar._select_button.connect(
             'toggled', self._on_header_bar_toggled)
@@ -108,10 +108,17 @@ class BaseView(Gtk.Stack):
 
         self._init = False
         grilo.connect('ready', self._on_grilo_ready)
-        self._header_bar.connect('selection-mode-changed',
-                                 self._on_selection_mode_changed)
+        self._header_bar.connect(
+            'notify::selection-mode', self._on_selection_mode_changed)
         grilo.connect('changes-pending', self._on_changes_pending)
 
+        self.bind_property(
+            'selection-mode', self._selection_toolbar, 'visible',
+            GObject.BindingFlags.SYNC_CREATE)
+
+        self.bind_property(
+            'selection-mode', self._header_bar, 'selection-mode')
+
     @log
     def _on_changes_pending(self, data=None):
         pass
@@ -140,30 +147,25 @@ class BaseView(Gtk.Stack):
         self.selection_mode = button.get_active()
 
         if self.selection_mode:
-            self._header_bar.set_selection_mode(True)
-            self.player.actionbar.set_visible(False)
+            self.set_player_visible(False)
             select_toolbar = self._selection_toolbar
-            select_toolbar.actionbar.set_visible(True)
-            select_toolbar._add_to_playlist_button.set_sensitive(False)
+            select_toolbar.add_to_playlist_button.set_sensitive(False)
         else:
-            self._header_bar.set_selection_mode(False)
-            track_playing = self.player.currentTrack is not None
-            self.player.actionbar.set_visible(track_playing)
-            self._selection_toolbar.actionbar.set_visible(False)
+            self.set_player_visible(self.player.current_song is not None)
             self.unselect_all()
 
     @log
     def _on_cancel_button_clicked(self, button):
         self.unselect_all()
-        self._header_bar.set_selection_mode(False)
+        self.props.selection_mode = False
 
     @log
     def _on_grilo_ready(self, data=None):
-        if (self._header_bar.get_stack().get_visible_child() == self
+        if (self._header_bar.props.stack.props.visible_child == self
                 and not self._init):
             self._populate()
-        self._header_bar.get_stack().connect('notify::visible-child',
-                                             self._on_headerbar_visible)
+        self._header_bar.props.stack.connect(
+            'notify::visible-child', self._on_headerbar_visible)
 
     @log
     def _on_headerbar_visible(self, widget, param):
@@ -182,19 +184,11 @@ class BaseView(Gtk.Stack):
     def update_header_from_selection(self, n_items):
         """Updates header during item selection."""
         select_toolbar = self._selection_toolbar
-        select_toolbar._add_to_playlist_button.set_sensitive(n_items > 0)
-        if n_items > 0:
-            self._header_bar._selection_menu_label.set_text(
-                ngettext("Selected {} item",
-                         "Selected {} items",
-                         n_items).format(n_items))
-        else:
-            self._header_bar._selection_menu_label.set_text(
-                _("Click on items to select them"))
+        select_toolbar.add_to_playlist_button.set_sensitive(n_items > 0)
+        self._header_bar.props.items_selected = n_items
 
     @log
     def _populate(self, data=None):
-        self._init = True
         self.populate()
 
     @log
@@ -209,30 +203,6 @@ class BaseView(Gtk.Stack):
     def _retrieval_finished(self, klass):
         self.model[klass.iter][4] = klass.pixbuf
 
-    @log
-    def _add_item(self, source, param, item, remaining=0, data=None):
-        if not item:
-            if remaining == 0:
-                self._view.set_model(self.model)
-                self._window.notifications_popup.pop_loading()
-                self._view.show()
-            return
-        self._offset += 1
-        artist = utils.get_artist_name(item)
-        title = utils.get_media_title(item)
-
-        itr = self.model.append(None)
-
-        self.model[itr][0, 1, 2, 3, 5, 7, 9] = [
-            str(item.get_id()),
-            '',
-            title,
-            artist,
-            item,
-            0,
-            False
-        ]
-
     @log
     def _add_list_renderers(self):
         pass
@@ -269,17 +239,22 @@ class BaseView(Gtk.Stack):
 
         if count > 0:
             select_toolbar = self._selection_toolbar
-            select_toolbar._add_to_playlist_button.set_sensitive(True)
+            select_toolbar.add_to_playlist_button.set_sensitive(True)
 
         self.update_header_from_selection(count)
-        self._view.queue_draw()
 
     @log
     def unselect_all(self):
         """Unselects all the selected songs."""
         self._set_selection(False)
         select_toolbar = self._selection_toolbar
-        select_toolbar._add_to_playlist_button.set_sensitive(False)
-        self._header_bar._selection_menu_label.set_text(
-            _("Click on items to select them"))
-        self.queue_draw()
+        select_toolbar.add_to_playlist_button.set_sensitive(False)
+        self._header_bar.props.items_selected = 0
+
+    @log
+    def set_player_visible(self, visible):
+        """Set PlayWidget action visibility
+
+        :param bool visible: Set actionbar visibility
+        """
+        self._window.player_toolbar.set_visible(visible)
