@@ -31,9 +31,9 @@ from gnomemusic import log
 from gnomemusic.player import DiscoveryStatus
 from gnomemusic.playlists import Playlists
 from gnomemusic.query import Query
-from gnomemusic.toolbar import ToolbarState
 from gnomemusic.utils import View
 from gnomemusic.views.baseview import BaseView
+from gnomemusic.widgets.headerbar import HeaderBar
 from gnomemusic.widgets.albumwidget import AlbumWidget
 from gnomemusic.widgets.artistalbumswidget import ArtistAlbumsWidget
 import gnomemusic.utils as utils
@@ -82,9 +82,7 @@ class SearchView(BaseView):
     @log
     def _no_music_found_callback(self, view):
         # FIXME: call into private members
-        self._window._stack.set_visible_child_name('emptysearch')
-        emptysearch = self._window._stack.get_child_by_name('emptysearch')
-        emptysearch._artist_albums_widget = self._artist_albums_widget
+        self._window._stack.set_visible_child_name("emptyview")
 
     @log
     def _back_button_clicked(self, widget, data=None):
@@ -98,7 +96,7 @@ class SearchView(BaseView):
                 self._window.views[View.ALBUM]._grid)
 
         self.set_visible_child(self._grid)
-        self._window.toolbar.set_state(ToolbarState.MAIN)
+        self._header_bar.props.state = HeaderBar.State.MAIN
 
     @log
     def _on_item_activated(self, widget, id, path):
@@ -112,38 +110,38 @@ class SearchView(BaseView):
             return
 
         _iter = self.model.get_iter(child_path)
-        if self.model[_iter][11] == 'album':
+        if self.model[_iter][12] == 'album':
             title = self.model[_iter][2]
             artist = self.model[_iter][3]
             item = self.model[_iter][5]
 
             self._album_widget.update(
-                artist, title, item, self._header_bar, self._selection_toolbar)
-            self._header_bar.set_state(ToolbarState.SEARCH_VIEW)
+                item, self._header_bar, self._selection_toolbar)
+            self._header_bar.props.state = HeaderBar.State.SEARCH
 
-            self._header_bar.header_bar.set_title(title)
-            self._header_bar.header_bar.sub_title = artist
+            self._header_bar.props.title = title
+            self._header_bar.props.subtitle = artist
             self.set_visible_child(self._album_widget)
             self._header_bar.searchbar.reveal(False)
-        elif self.model[_iter][11] == 'artist':
+        elif self.model[_iter][12] == 'artist':
             artist = self.model[_iter][2]
             albums = self._artists[artist.casefold()]['albums']
 
             self._artist_albums_widget = ArtistAlbumsWidget(
-                artist, albums, self.player, self._header_bar,
+                artist, albums, self.player, self, self._header_bar,
                 self._selection_toolbar, self._window, True)
             self.add(self._artist_albums_widget)
             self._artist_albums_widget.show()
 
-            self._header_bar.set_state(ToolbarState.SEARCH_VIEW)
-            self._header_bar.header_bar.set_title(artist)
+            self._header_bar.props.state = HeaderBar.State.SEARCH
+            self._header_bar.props.title = artist
             self.set_visible_child(self._artist_albums_widget)
             self._header_bar.searchbar.reveal(False)
-        elif self.model[_iter][11] == 'song':
-            if self.model[_iter][12] != DiscoveryStatus.FAILED:
+        elif self.model[_iter][12] == 'song':
+            if self.model[_iter][11] != DiscoveryStatus.FAILED:
                 c_iter = self._songs_model.convert_child_iter_to_iter(_iter)[1]
                 self.player.set_playlist(
-                    'Search Results', None, self._songs_model, c_iter, 12)
+                    'Search Results', None, self._songs_model, c_iter)
                 self.player.set_playing(True)
         else:  # Headers
             if self._view.get_generic_view().row_expanded(path):
@@ -215,7 +213,7 @@ class SearchView(BaseView):
                 self.emit('no-music-found')
 
         # We need to remember the view before the search view
-        emptysearchview = self._window.views[View.EMPTY_SEARCH]
+        emptysearchview = self._window.views[View.EMPTY]
         if (self._window.curr_view != emptysearchview
                 and self._window.prev_view != emptysearchview):
             self.previous_view = self._window.prev_view
@@ -235,13 +233,13 @@ class SearchView(BaseView):
         group = 3
         try:
             group = {'album': 0, 'artist': 1, 'song': 2}[category]
-        except:
+        except KeyError:
             pass
 
         _iter = None
         if category == 'album':
             _iter = self.model.insert_with_values(
-                self._head_iters[group], -1, [0, 2, 3, 5, 9, 11],
+                self._head_iters[group], -1, [0, 2, 3, 5, 9, 12],
                 [str(item.get_id()), title, artist, item, 2,
                  category])
         elif category == 'song':
@@ -251,13 +249,13 @@ class SearchView(BaseView):
             else:
                 fav = item.get_favourite()
             _iter = self.model.insert_with_values(
-                self._head_iters[group], -1, [0, 2, 3, 5, 9, 11],
+                self._head_iters[group], -1, [0, 2, 3, 5, 9, 12],
                 [str(item.get_id()), title, artist, item, fav,
                  category])
         else:
             if not artist.casefold() in self._artists:
                 _iter = self.model.insert_with_values(
-                    self._head_iters[group], -1, [0, 2, 5, 9, 11],
+                    self._head_iters[group], -1, [0, 2, 5, 9, 12],
                     [str(item.get_id()), artist, item, 2,
                      category])
                 self._artists[artist.casefold()] = {
@@ -304,7 +302,7 @@ class SearchView(BaseView):
             pixbuf_renderer, self._on_list_widget_pixbuf_renderer, None)
         cols[0].add_attribute(pixbuf_renderer, 'surface', 13)
 
-        self._star_handler.add_star_renderers(list_widget, cols[0])
+        self._star_handler.add_star_renderers(cols[0])
         cells = cols[0].get_cells()
 
         # With the bugfix in libgd 9117650bda, the search results
@@ -346,7 +344,7 @@ class SearchView(BaseView):
     @log
     def populate(self):
         self._init = True
-        self._header_bar.set_state(ToolbarState.MAIN)
+        self._header_bar.props.state = HeaderBar.State.MAIN
 
     @log
     def get_selected_songs(self, callback):
@@ -373,7 +371,7 @@ class SearchView(BaseView):
         self._albums_selected = [
             self.model[child_path][5]
             for child_path in paths
-            if self.model[child_path][11] == 'album']
+            if self.model[child_path][12] == 'album']
 
         if len(self._albums_selected):
             self._get_selected_albums_songs()
@@ -405,7 +403,7 @@ class SearchView(BaseView):
             for child_path in [
                 self._filter_model.convert_path_to_child_path(path)
                 for path in self._view.get_selection()]
-            if self.model[child_path][11] == 'artist']
+            if self.model[child_path][12] == 'artist']
 
         self._artists_albums_selected = []
         for artist in artists_selected:
@@ -442,7 +440,7 @@ class SearchView(BaseView):
             for child_path in [
                 self._filter_model.convert_path_to_child_path(path)
                 for path in self._view.get_selection()]
-            if self.model[child_path][11] == 'song'])
+            if self.model[child_path][12] == 'song'])
         self._items_selected_callback(self._items_selected)
 
     @log
@@ -490,8 +488,8 @@ class SearchView(BaseView):
             GObject.TYPE_STRING,
             GObject.TYPE_INT,
             GObject.TYPE_BOOLEAN,
+            GObject.TYPE_INT,       # discovery status
             GObject.TYPE_STRING,    # type
-            GObject.TYPE_INT,
             object                  # album art surface
         )
 
