@@ -64,7 +64,7 @@ class SongsView(BaseView):
         self._add_list_renderers()
 
         self.player = player
-        self.player.connect('playlist-item-changed', self.update_model)
+        self.player.connect('song-changed', self.update_model)
 
     @log
     def _on_changes_pending(self, data=None):
@@ -169,7 +169,7 @@ class SongsView(BaseView):
 
         col = Gtk.TreeViewColumn()
         col.set_expand(False)
-        self._star_handler.add_star_renderers(list_widget, col)
+        self._star_handler.add_star_renderers(col)
         list_widget.append_column(col)
 
         duration_renderer = Gd.StyledTextRenderer(xpad=32, xalign=1.0)
@@ -221,7 +221,7 @@ class SongsView(BaseView):
             cell.set_property('text', utils.get_album_title(item))
 
     def _on_list_widget_icon_render(self, col, cell, model, itr, data):
-        track_uri = self.player.current_track_uri
+        track_uri = self.player.url
         if not track_uri:
             cell.set_visible(False)
             return
