@@ -22,13 +22,15 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-from gettext import gettext as _, ngettext
+from gettext import ngettext
 from gi.repository import GdkPixbuf, GLib, GObject, Gtk
 
 from gnomemusic import log
 from gnomemusic.albumartcache import Art, ArtImage
 from gnomemusic.grilo import grilo
+from gnomemusic.gstplayer import Playback
 from gnomemusic.widgets.disclistboxwidget import DiscBox, DiscListBox
+from gnomemusic.widgets.songwidget import SongWidget
 import gnomemusic.utils as utils
 
 
@@ -53,23 +55,9 @@ class AlbumWidget(Gtk.EventBox):
         """
         super().__init__()
 
-        self._model = Gtk.ListStore(
-            GObject.TYPE_STRING,   # title
-            GObject.TYPE_STRING,
-            GObject.TYPE_STRING,
-            GObject.TYPE_STRING,
-            GdkPixbuf.Pixbuf,      # icon
-            GObject.TYPE_OBJECT,   # song object
-            GObject.TYPE_BOOLEAN,  # item selected
-            GObject.TYPE_STRING,
-            GObject.TYPE_BOOLEAN,
-            GObject.TYPE_INT,      # icon shown
-            GObject.TYPE_BOOLEAN,
-            GObject.TYPE_INT
-        )
-
         self._songs = []
 
+        self._parent_view = parent_view
         self._player = player
         self._iter_to_clean = None
 
@@ -77,6 +65,7 @@ class AlbumWidget(Gtk.EventBox):
 
         self._builder = Gtk.Builder()
         self._builder.add_from_resource('/org/gnome/Music/AlbumWidget.ui')
+        self._create_model()
         self._album = None
         self._header_bar = None
         self._selection_mode_allowed = True
@@ -115,24 +104,39 @@ class AlbumWidget(Gtk.EventBox):
         self._header_bar._select_button.clicked()
 
     @log
-    def update(self, artist, album, item, header_bar, selection_toolbar):
+    def _create_model(self):
+        """Create the ListStore model for this widget."""
+        self._model = Gtk.ListStore(
+            GObject.TYPE_STRING,  # title
+            GObject.TYPE_STRING,
+            GObject.TYPE_STRING,
+            GObject.TYPE_STRING,
+            GdkPixbuf.Pixbuf,    # icon
+            GObject.TYPE_OBJECT,  # song object
+            GObject.TYPE_BOOLEAN,  # item selected
+            GObject.TYPE_STRING,
+            GObject.TYPE_BOOLEAN,
+            GObject.TYPE_INT,  # icon shown
+            GObject.TYPE_BOOLEAN,
+            GObject.TYPE_INT
+        )
+
+    @log
+    def update(self, item, header_bar, selection_toolbar):
         """Update the album widget.
 
-        :param str artist: The artist name
-        :param str album: The album name
         :param item: The grilo media item
         :param header_bar: The header bar object
         :param selection_toolbar: The selection toolbar object
         """
         # reset view
         self._songs = []
-        self._model.clear()
+        self._create_model()
         for widget in self._disc_listbox.get_children():
             self._disc_listbox.remove(widget)
 
         self.selection_toolbar = selection_toolbar
         self._header_bar = header_bar
-        self._album = album
         self._duration = 0
         art = ArtImage(Art.Size.LARGE, item)
         art.image = self._builder.get_object('cover')
@@ -143,21 +147,19 @@ class AlbumWidget(Gtk.EventBox):
         header_bar._cancel_button.connect(
             'clicked', self._on_header_cancel_button_clicked)
 
-        # FIXME: use utils
-        escaped_artist = GLib.markup_escape_text(artist)
-        escaped_album = GLib.markup_escape_text(album)
-        self._builder.get_object('artist_label').set_markup(escaped_artist)
-        self._builder.get_object('title_label').set_markup(escaped_album)
+        self._album = utils.get_album_title(item)
+        self._builder.get_object('artist_label').set_text(
+            utils.get_artist_name(item))
+        self._builder.get_object('title_label').set_text(self._album)
 
-        if (item.get_creation_date()):
-            self._builder.get_object('released_label_info').set_text(
-                str(item.get_creation_date().get_year()))
-        else:
-            self._builder.get_object('released_label_info').set_text('----')
+        year = utils.get_media_year(item)
+        if not year:
+            year = '----'
+        self._builder.get_object('released_label_info').set_text(year)
 
         self._set_composer_label(item)
 
-        self._player.connect('playlist-item-changed', self._update_model)
+        self._player.connect('song-changed', self._update_model)
 
     @log
     def _set_composer_label(self, item):
@@ -171,50 +173,50 @@ class AlbumWidget(Gtk.EventBox):
         self._composer_label.set_visible(show)
         self._composer_info.set_visible(show)
 
+    @log
+    def _set_duration_label(self):
+        mins = (self._duration // 60) + 1
+        self._builder.get_object('running_length_label_info').set_text(
+            ngettext("{} minute", "{} minutes", mins).format(mins))
+
     @log
     def _on_selection_changed(self, widget):
         items = self._disc_listbox.get_selected_items()
-        self.selection_toolbar._add_to_playlist_button.set_sensitive(
+        self.selection_toolbar.add_to_playlist_button.set_sensitive(
             len(items) > 0)
-        if len(items) > 0:
-            self._header_bar._selection_menu_label.set_text(
-                ngettext("Selected %d item", "Selected %d items",
-                         len(items)) % len(items))
-        else:
-            self._header_bar._selection_menu_label.set_text(
-                _("Click on items to select them"))
+        self._header_bar.items_selected = len(items)
 
     @log
     def _on_header_cancel_button_clicked(self, button):
         """Cancel selection mode callback."""
-        self._disc_listbox.set_selection_mode(False)
-        self._header_bar.set_selection_mode(False)
-        self._header_bar.header_bar.title = self._album
+        self._disc_listbox.props.selection_mode = False
+        self._header_bar.props.selection_mode = False
+        self._header_bar.props.title = self._album
 
     @log
     def _on_header_select_button_toggled(self, button):
         """Selection mode button clicked callback."""
         if button.get_active():
             self._selection_mode = True
-            self._disc_listbox.set_selection_mode(True)
-            self._header_bar.set_selection_mode(True)
-            self._player.actionbar.set_visible(False)
-            self._header_bar.header_bar.set_custom_title(
-                self._header_bar._selection_menu_button)
+            self._disc_listbox.props.selection_mode = True
+            self._header_bar.props.selection_mode = True
+            self._parent_view.set_player_visible(False)
         else:
             self._selection_mode = False
-            self._disc_listbox.set_selection_mode(False)
-            self._header_bar.set_selection_mode(False)
-            if(self._player.get_playback_status() != 2):
-                self._player.actionbar.set_visible(True)
+            self._disc_listbox.props.selection_mode = False
+            self._header_bar.props.selection_mode = False
+            if self._player.get_playback_status() != Playback.STOPPED:
+                self._parent_view.set_player_visible(True)
 
     @log
     def _create_disc_box(self, disc_nr, disc_songs):
         disc_box = DiscBox(self._model)
         disc_box.set_songs(disc_songs)
         disc_box.set_disc_number(disc_nr)
-        disc_box.set_columns(1)
-        disc_box.show_song_numbers(False)
+        disc_box.props.columns = 1
+        disc_box.props.show_durations = True
+        disc_box.props.show_favorites = True
+        disc_box.props.show_song_numbers = False
         disc_box.connect('song-activated', self._song_activated)
         disc_box.connect('selection-toggle', self._selection_mode_toggled)
 
@@ -230,11 +232,8 @@ class AlbumWidget(Gtk.EventBox):
 
     @log
     def _song_activated(self, widget, song_widget):
-        if not song_widget.can_be_played:
-            return
-
         if self._selection_mode:
-            song_widget.check_button.toggled()
+            song_widget.props.selected = not song_widget.props.selected
             return
 
         self._player.stop()
@@ -243,12 +242,6 @@ class AlbumWidget(Gtk.EventBox):
         self._player.set_playing(True)
         return True
 
-    @log
-    def _set_duration_label(self):
-        duration = self._duration // 60 + 1
-        self._builder.get_object('running_length_label_info').set_text(
-            _("%d min") % duration)
-
     @log
     def add_item(self, source, prefs, song, remaining, data=None):
         """Add a song to the item to album list.
@@ -276,16 +269,11 @@ class AlbumWidget(Gtk.EventBox):
             disc = self._create_disc_box(disc_nr, discs[disc_nr])
             self._disc_listbox.add(disc)
             if len(discs) == 1:
-                disc.show_disc_label(False)
+                disc.props.show_disc_label = False
 
         if remaining == 0:
-            if self._player.running_playlist('Album', self._album):
-                current_path = self._player.currentTrack.get_path()
-                current_track = self._player.playlist.get_iter(current_path)
-                self._update_model(
-                    self._player, self._player.playlist, current_track)
-            else:
-                self._set_duration_label()
+            self._set_duration_label()
+
             self.show_all()
 
     @log
@@ -304,28 +292,23 @@ class AlbumWidget(Gtk.EventBox):
         self._duration = 0
 
         song_passed = False
-        _iter = self._model.get_iter_first()
+        _iter = playlist.get_iter_first()
 
         while _iter:
-            song = self._model[_iter][5]
+            song = playlist[_iter][player.Field.SONG]
             song_widget = song.song_widget
             self._duration += song.get_duration()
-            escaped_title = GLib.markup_escape_text(
-                utils.get_media_title(song))
-            if song.get_id() == current_song.get_id():
-                song_widget.now_playing_sign.show()
-                song_widget.title.set_markup("<b>{}</b>".format(escaped_title))
+
+            if (song == current_song):
+                song_widget.props.state = SongWidget.State.PLAYING
                 song_passed = True
             elif (song_passed):
-                song_widget.now_playing_sign.hide()
-                song_widget.title.set_markup(
-                    "<span>{}</span>".format(escaped_title))
+                # Counter intuitive, but this is due to call order.
+                song_widget.props.state = SongWidget.State.UNPLAYED
             else:
-                song_widget.now_playing_sign.hide()
-                song_widget.title.set_markup(
-                    "<span color=\'grey\'>{}</span>".format(escaped_title))
+                song_widget.props.state = SongWidget.State.PLAYED
 
-            _iter = self._model.iter_next(_iter)
+            _iter = playlist.iter_next(_iter)
 
         self._set_duration_label()
 
