@@ -24,12 +24,11 @@
 
 import logging
 
-from gettext import gettext as _, ngettext
-from gi.repository import GLib, GObject, Gtk
+from gi.repository import GObject, Gtk
 
 from gnomemusic import log
 from gnomemusic.widgets.artistalbumwidget import ArtistAlbumWidget
-import gnomemusic.utils as utils
+from gnomemusic.widgets.songwidget import SongWidget
 
 logger = logging.getLogger(__name__)
 
@@ -46,12 +45,13 @@ class ArtistAlbumsWidget(Gtk.Box):
         return '<ArtistAlbumsWidget>'
 
     @log
-    def __init__(self, artist, albums, player, header_bar, selection_toolbar,
-                 window, selection_mode_allowed=False):
+    def __init__(self, artist, albums, player, parent_view, header_bar,
+                 selection_toolbar, window, selection_mode_allowed=False):
         super().__init__(orientation=Gtk.Orientation.VERTICAL)
         self._player = player
         self.artist = artist
         self._window = window
+        self._parent_view = parent_view
         self._selection_mode = False
         self._selection_mode_allowed = selection_mode_allowed
         self._selection_toolbar = selection_toolbar
@@ -92,7 +92,7 @@ class ArtistAlbumsWidget(Gtk.Box):
                 is_last_album = True
             self._add_album(album, is_last_album)
 
-        self._player.connect('playlist-item-changed', self._update_model)
+        self._player.connect('song-changed', self._update_model)
 
     @log
     def _create_model(self):
@@ -119,11 +119,10 @@ class ArtistAlbumsWidget(Gtk.Box):
 
     @log
     def _add_album(self, album, is_last_album=False):
-        widget = ArtistAlbumWidget(album, self._player, self._model,
-                                   self._header_bar,
-                                   self._selection_mode_allowed,
-                                   self._songs_grid_size_group,
-                                   self._cover_size_group)
+        widget = ArtistAlbumWidget(
+            album, self._player, self._model, self._parent_view,
+            self._header_bar, self._selection_mode_allowed,
+            self._songs_grid_size_group, self._cover_size_group)
         self._cover_size_group.add_widget(widget.cover_stack._stack)
 
         self._album_box.pack_start(widget, False, False, 0)
@@ -146,24 +145,15 @@ class ArtistAlbumsWidget(Gtk.Box):
             song = playlist[itr][player.Field.SONG]
             song_widget = song.song_widget
 
-            if not song_widget.can_be_played:
-                itr = playlist.iter_next(itr)
-                continue
-
-            escaped_title = GLib.markup_escape_text(
-                utils.get_media_title(song))
-
             if (song == current_song):
-                song_widget.now_playing_sign.show()
-                song_widget.title.set_markup('<b>%s</b>' % escaped_title)
+                song_widget.props.state = SongWidget.State.PLAYING
                 song_passed = True
             elif (song_passed):
-                song_widget.now_playing_sign.hide()
-                song_widget.title.set_markup('<span>%s</span>' % escaped_title)
+                # Counter intuitive, but this is due to call order.
+                song_widget.props.state = SongWidget.State.UNPLAYED
             else:
-                song_widget.now_playing_sign.hide()
-                song_widget.title.set_markup(
-                    '<span color=\'grey\'>%s</span>' % escaped_title)
+                song_widget.props.state = SongWidget.State.PLAYED
+
             itr = playlist.iter_next(itr)
 
         return False
@@ -175,11 +165,8 @@ class ArtistAlbumsWidget(Gtk.Box):
         while itr:
             song = self._model[itr][5]
             song_widget = song.song_widget
-            escaped_title = GLib.markup_escape_text(
-                utils.get_media_title(song))
-            if song_widget.can_be_played:
-                song_widget.now_playing_sign.hide()
-            song_widget.title.set_markup('<span>%s</span>' % escaped_title)
+            song_widget.props.state = SongWidget.State.UNPLAYED
+
             itr = self._model.iter_next(itr)
 
         return False
@@ -216,16 +203,10 @@ class ArtistAlbumsWidget(Gtk.Box):
             if row[6]:
                 selected_items += 1
 
-        add_button = self._selection_toolbar._add_to_playlist_button
+        add_button = self._selection_toolbar.add_to_playlist_button
         add_button.set_sensitive(selected_items > 0)
 
-        menu_label = self._header_bar._selection_menu_label
-        if selected_items > 0:
-            menu_label.set_text(ngettext("Selected %d item",
-                                         "Selected %d items",
-                                         selected_items) % selected_items)
-        else:
-            menu_label.set_text(_("Click on items to select them"))
+        self._header_bar.selected_items = selected_items
 
     @log
     def select_all(self):
