@@ -33,28 +33,26 @@ from gi.repository import Gtk, Gdk, Gio, GLib
 from gettext import gettext as _
 
 from gnomemusic import log
-from gnomemusic import TrackerWrapper
-from gnomemusic.toolbar import Toolbar, ToolbarState
-from gnomemusic.player import Player, SelectionToolbar, RepeatType
+from gnomemusic.player import Player, RepeatType
 from gnomemusic.query import Query
 from gnomemusic.utils import View
 from gnomemusic.views.albumsview import AlbumsView
 from gnomemusic.views.artistsview import ArtistsView
 from gnomemusic.views.emptyview import EmptyView
-from gnomemusic.views.emptysearchview import EmptySearchView
-from gnomemusic.views.initialstateview import InitialStateView
 from gnomemusic.views.searchview import SearchView
 from gnomemusic.views.songsview import SongsView
 from gnomemusic.views.playlistview import PlaylistView
+from gnomemusic.widgets.headerbar import HeaderBar
 from gnomemusic.widgets.notificationspopup import NotificationsPopup
+from gnomemusic.widgets.playertoolbar import PlayerToolbar
 from gnomemusic.widgets.playlistdialog import PlaylistDialog
+from gnomemusic.widgets.selectiontoolbar import SelectionToolbar
 from gnomemusic.playlists import Playlists
 from gnomemusic.grilo import grilo
 
 import logging
 logger = logging.getLogger(__name__)
 
-tracker = TrackerWrapper().tracker
 playlists = Playlists.get_default()
 
 
@@ -77,7 +75,7 @@ class Window(Gtk.ApplicationWindow):
         self.add_action(select_none)
 
         self.set_size_request(200, 100)
-        self.set_default_icon_name('gnome-music')
+        self.set_default_icon_name('org.gnome.Music')
 
         self.prev_view = None
         self.curr_view = None
@@ -111,23 +109,20 @@ class Window(Gtk.ApplicationWindow):
     def _on_changes_pending(self, data=None):
         # FIXME: This is not working right.
         def songs_available_cb(available):
-            if available:
-                if self.views[View.ALBUM] == self.views[-1]:
-                    view = self.views.pop()
-                    view.destroy()
-                    self._switch_to_player_view()
-                    self.toolbar._search_button.set_sensitive(True)
-                    self.toolbar._select_button.set_sensitive(True)
-                    self.toolbar.show_stack()
-            elif (self.toolbar.selection_mode is False
-                    and len(self.views) != 1):
+            view_count = len(self.views)
+            if (available
+                    and view_count == 1):
+                self._switch_to_player_view()
+            elif (not available
+                    and not self.headerbar.props.selection_mode
+                    and view_count != 1):
                 self._stack.disconnect(self._on_notify_model_id)
                 self.disconnect(self._key_press_event_id)
-                view_count = len(self.views)
-                for i in range(0, view_count):
+
+                for i in range(View.ALBUM, view_count):
                     view = self.views.pop()
                     view.destroy()
-                self.toolbar.hide_stack()
+
                 self._switch_to_empty_view()
 
         grilo.songs_available(songs_available_cb)
@@ -204,18 +199,19 @@ class Window(Gtk.ApplicationWindow):
         if "Play" in response:
             self.player.play_pause()
         elif "Stop" in response:
-            self.player.Stop()
+            self.player.stop()
         elif "Next" in response:
-            self.player.play_next()
+            self.player.next()
         elif "Previous" in response:
-            self.player.play_previous()
+            self.player.previous()
 
     @log
     def _setup_view(self):
         self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         self.player = Player(self)
+        self.player_toolbar = PlayerToolbar(self.player)
         self.selection_toolbar = SelectionToolbar()
-        self.toolbar = Toolbar()
+        self.headerbar = HeaderBar()
         self.views = [None] * len(View)
         self._stack = Gtk.Stack(
             transition_type=Gtk.StackTransitionType.CROSSFADE,
@@ -224,17 +220,24 @@ class Window(Gtk.ApplicationWindow):
             visible=True,
             can_focus=False)
 
+        # Create only the empty view at startup
+        # if no music, switch to empty view and hide stack
+        # if some music is available, populate stack with mainviews,
+        # show stack and set empty_view to empty_search_view
+        self.views[View.EMPTY] = EmptyView()
+        self._stack.add_named(self.views[View.EMPTY], "emptyview")
+
         # Add the 'background' styleclass so it properly hides the
         # bottom line of the searchbar
         self._stack.get_style_context().add_class('background')
 
         self._overlay = Gtk.Overlay(child=self._stack)
-        self._overlay.add_overlay(self.toolbar.dropdown)
-        self.set_titlebar(self.toolbar.header_bar)
-        self._box.pack_start(self.toolbar.searchbar, False, False, 0)
+        self._overlay.add_overlay(self.headerbar.dropdown)
+        self.set_titlebar(self.headerbar)
+        self._box.pack_start(self.headerbar.searchbar, False, False, 0)
         self._box.pack_start(self._overlay, True, True, 0)
-        self._box.pack_start(self.player.actionbar, False, False, 0)
-        self._box.pack_start(self.selection_toolbar.actionbar, False, False, 0)
+        self._box.pack_start(self.player_toolbar, False, False, 0)
+        self._box.pack_start(self.selection_toolbar, False, False, 0)
         self.add(self._box)
 
         def songs_available_cb(available):
@@ -249,64 +252,69 @@ class Window(Gtk.ApplicationWindow):
         else:
             self._switch_to_empty_view()
 
-        self.toolbar._search_button.connect('toggled', self._on_search_toggled)
-        self.toolbar.connect('selection-mode-changed', self._on_selection_mode_changed)
-        self.selection_toolbar._add_to_playlist_button.connect(
+        self.headerbar._search_button.connect(
+            'toggled', self._on_search_toggled)
+        self.headerbar.connect(
+            'notify::selection-mode', self._on_selection_mode_changed)
+        self.selection_toolbar.add_to_playlist_button.connect(
             'clicked', self._on_add_to_playlist_button_clicked)
 
-        self.toolbar.set_state(ToolbarState.MAIN)
-        self.toolbar.header_bar.show()
+        self.headerbar.props.state = HeaderBar.State.MAIN
+        self.headerbar.show()
         self._overlay.show()
-        self.player.actionbar.show_all()
+        self.player_toolbar.show_all()
         self._box.show()
         self.show()
 
     @log
     def _switch_to_empty_view(self):
         did_initial_state = self.settings.get_boolean('did-initial-state')
-        view_class = None
-        if did_initial_state:
-            view_class = EmptyView
-        else:
-            view_class = InitialStateView
-        self.views[View.ALBUM] = view_class(self, self.player)
 
-        self._stack.add_titled(self.views[View.ALBUM], _("Empty"), _("Empty"))
-        self.toolbar._search_button.set_sensitive(False)
-        self.toolbar._select_button.set_sensitive(False)
+        if did_initial_state:
+            self.views[View.EMPTY].props.state = EmptyView.State.EMPTY
+        else:
+            self.views[View.EMPTY].props.state = EmptyView.State.INITIAL
+
+        self.headerbar.props.state = HeaderBar.State.EMPTY
 
     @log
     def _switch_to_player_view(self):
         self.settings.set_boolean('did-initial-state', True)
-        self._on_notify_model_id = self._stack.connect('notify::visible-child', self._on_notify_mode)
+        self._on_notify_model_id = self._stack.connect(
+            'notify::visible-child', self._on_notify_mode)
         self.connect('destroy', self._notify_mode_disconnect)
-        self._key_press_event_id = self.connect('key_press_event', self._on_key_press)
+        self._key_press_event_id = self.connect(
+            'key_press_event', self._on_key_press)
 
         self.views[View.ALBUM] = AlbumsView(self, self.player)
         self.views[View.ARTIST] = ArtistsView(self, self.player)
         self.views[View.SONG] = SongsView(self, self.player)
         self.views[View.PLAYLIST] = PlaylistView(self, self.player)
         self.views[View.SEARCH] = SearchView(self, self.player)
-        self.views[View.EMPTY_SEARCH] = EmptySearchView(self, self.player)
 
-        for i in self.views:
+        # empty view has already been created in self._setup_view starting at
+        # View.ALBUM
+        # empty view state is changed once album view is visible to prevent it
+        # from being displayed during startup
+        for i in self.views[View.ALBUM:]:
             if i.title:
                 self._stack.add_titled(i, i.name, i.title)
             else:
                 self._stack.add_named(i, i.name)
-
-        self.toolbar.set_stack(self._stack)
-        self.toolbar.searchbar.show()
-        self.toolbar.dropdown.show()
-
-        for i in self.views:
             GLib.idle_add(i.populate)
 
+        self._stack.set_visible_child(self.views[View.ALBUM])
+        self.views[View.EMPTY].props.state = EmptyView.State.SEARCH
+        self.headerbar.props.state = HeaderBar.State.MAIN
+        self.headerbar.props.stack = self._stack
+        self.headerbar.searchbar.show()
+        self.headerbar.dropdown.show()
+
     @log
     def _select_all(self, action=None, param=None):
-        if not self.toolbar.selection_mode:
+        if not self.headerbar.props.selection_mode:
             return
-        if self.toolbar._state == ToolbarState.MAIN:
+        if self.headerbar.props.state == HeaderBar.State.MAIN:
             view = self._stack.get_visible_child()
         else:
             view = self._stack.get_visible_child().get_visible_child()
@@ -315,9 +323,9 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _select_none(self, action=None, param=None):
-        if not self.toolbar.selection_mode:
+        if not self.headerbar.props.selection_mode:
             return
-        if self.toolbar._state == ToolbarState.MAIN:
+        if self.headerbar.props.state == HeaderBar.State.MAIN:
             view = self._stack.get_visible_child()
             view.unselect_all()
         else:
@@ -343,8 +351,8 @@ class Window(Gtk.ApplicationWindow):
             # Open search bar on Ctrl + F
             if ((event.keyval == Gdk.KEY_f and modifiers == control_mask)
                     and not self.views[View.PLAYLIST].rename_active
-                    and self.toolbar._state != ToolbarState.SEARCH_VIEW):
-                self.toolbar.searchbar.toggle()
+                    and self.headerbar.props.state != HeaderBar.State.SEARCH):
+                self.headerbar.searchbar.toggle()
             # Play / Pause on Ctrl + SPACE
             if (event.keyval == Gdk.KEY_space
                     and modifiers == control_mask):
@@ -352,11 +360,11 @@ class Window(Gtk.ApplicationWindow):
             # Play previous on Ctrl + B
             if (event.keyval == Gdk.KEY_b
                     and modifiers == control_mask):
-                self.player.play_previous()
+                self.player.previous()
             # Play next on Ctrl + N
             if (event.keyval == Gdk.KEY_n
                     and modifiers == control_mask):
-                self.player.play_next()
+                self.player.next()
             # Toggle repeat on Ctrl + R
             if (event.keyval == Gdk.KEY_r
                     and modifiers == control_mask):
@@ -374,7 +382,7 @@ class Window(Gtk.ApplicationWindow):
             # Go back from Album view on Alt + Left
             if (event.keyval == Gdk.KEY_Left
                     and modifiers == mod1_mask):
-                self.toolbar.on_back_button_clicked()
+                self.headerbar._on_back_button_clicked()
             if ((event.keyval in [Gdk.KEY_1, Gdk.KEY_KP_1])
                     and modifiers == control_mask):
                 self._toggle_view(View.ALBUM)
@@ -393,13 +401,13 @@ class Window(Gtk.ApplicationWindow):
                 self.player.play_pause()
 
             if event.keyval == Gdk.KEY_AudioStop:
-                self.player.Stop()
+                self.player.stop()
 
             if event.keyval == Gdk.KEY_AudioPrev:
-                self.player.play_prev()
+                self.player.previous()
 
             if event.keyval == Gdk.KEY_AudioNext:
-                self.player.play_next()
+                self.player.next()
 
             child = self._stack.get_visible_child()
             if (event.keyval == Gdk.KEY_Delete
@@ -407,21 +415,21 @@ class Window(Gtk.ApplicationWindow):
                 self.views[View.PLAYLIST].remove_playlist()
             # Close search bar after Esc is pressed
             if event.keyval == Gdk.KEY_Escape:
-                self.toolbar.searchbar.reveal(False)
+                self.headerbar.searchbar.reveal(False)
                 # Also disable selection
-                if self.toolbar.selection_mode:
-                    self.toolbar.set_selection_mode(False)
+                if self.headerbar.props.selection_mode:
+                    self.headerbar.props.selection_mode = False
 
         # Open the search bar when typing printable chars.
         key_unic = Gdk.keyval_to_unicode(event.keyval)
-        if ((not self.toolbar.searchbar.get_search_mode()
+        if ((not self.headerbar.searchbar.get_search_mode()
                 and not event.keyval == Gdk.KEY_space)
                 and GLib.unichar_isprint(chr(key_unic))
                 and (modifiers == Gdk.ModifierType.SHIFT_MASK
                      or modifiers == 0)
                 and not self.views[View.PLAYLIST].rename_active
-                and self.toolbar._state != ToolbarState.SEARCH_VIEW):
-            self.toolbar.searchbar.reveal(True)
+                and self.headerbar.props.state != HeaderBar.State.SEARCH):
+            self.headerbar.searchbar.reveal(True)
 
     @log
     def do_button_release_event(self, event):
@@ -432,11 +440,11 @@ class Window(Gtk.ApplicationWindow):
         __, code = event.get_button()
         # Mouse button 8 is the navigation button
         if code == 8:
-            self.toolbar.on_back_button_clicked()
+            self.headerbar._on_back_button_clicked()
 
     @log
     def _notify_mode_disconnect(self, data=None):
-        self.player.Stop()
+        self.player.stop()
         self.notifications_popup.terminate_pending()
         self._stack.disconnect(self._on_notify_model_id)
 
@@ -448,7 +456,7 @@ class Window(Gtk.ApplicationWindow):
         # Switch to all albums view when we're clicking Albums
         if (self.curr_view == self.views[View.ALBUM]
                 and not (self.prev_view == self.views[View.SEARCH]
-                    or self.prev_view == self.views[View.EMPTY_SEARCH])):
+                    or self.prev_view == self.views[View.EMPTY])):
             self.curr_view.set_visible_child(self.curr_view._grid)
 
         # Slide out sidebar on switching to Artists or Playlists view
@@ -456,16 +464,17 @@ class Window(Gtk.ApplicationWindow):
            self.curr_view == self.views[View.PLAYLIST]:
             self.curr_view.stack.set_visible_child_name('dummy')
             self.curr_view.stack.set_visible_child_name('sidebar')
-        if self.curr_view != self.views[View.SEARCH] and self.curr_view != self.views[View.EMPTY_SEARCH]:
-            self.toolbar.searchbar.reveal(False)
+        if (self.curr_view != self.views[View.SEARCH]
+                and self.curr_view != self.views[View.EMPTY]):
+            self.headerbar.searchbar.reveal(False)
 
         # Disable the selection button for the EmptySearch and Playlist
         # view
         no_selection_mode = [
-            self.views[View.EMPTY_SEARCH],
+            self.views[View.EMPTY],
             self.views[View.PLAYLIST]
         ]
-        self.toolbar._select_button.set_sensitive(
+        self.headerbar._select_button.set_sensitive(
             self.curr_view not in no_selection_mode)
 
         # Disable renaming playlist if it was active when leaving
@@ -480,13 +489,13 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _on_search_toggled(self, button, data=None):
-        self.toolbar.searchbar.reveal(
+        self.headerbar.searchbar.reveal(
             button.get_active(), self.curr_view != self.views[View.SEARCH])
         if (not button.get_active()
                 and (self.curr_view == self.views[View.SEARCH]
-                    or self.curr_view == self.views[View.EMPTY_SEARCH])):
+                    or self.curr_view == self.views[View.EMPTY])):
             child = self.curr_view.get_visible_child()
-            if self.toolbar._state == ToolbarState.MAIN:
+            if self.headerbar.props.state == HeaderBar.State.MAIN:
                 # We should get back to the view before the search
                 self._stack.set_visible_child(
                     self.views[View.SEARCH].previous_view)
@@ -495,17 +504,18 @@ class Window(Gtk.ApplicationWindow):
                     and child != self.curr_view._artist_albums_widget):
                 self._stack.set_visible_child(self.views[View.ALBUM])
 
-            if self.toolbar.selection_mode:
-                self.toolbar.set_selection_mode(False)
+            if self.headerbar.props.selection_mode:
+                self.headerbar.props.selection_mode = False
 
     @log
     def _on_selection_mode_changed(self, widget, data=None):
-        if self.toolbar.selection_mode is False:
+        if self.headerbar.props.selection_mode == False:
             self._on_changes_pending()
         else:
             child = self._stack.get_visible_child()
             in_playlist = (child == self.views[View.PLAYLIST])
-            self.selection_toolbar._add_to_playlist_button.set_visible(not in_playlist)
+            self.selection_toolbar.add_to_playlist_button.set_visible(
+                not in_playlist)
 
     @log
     def _on_add_to_playlist_button_clicked(self, widget):
@@ -521,7 +531,15 @@ class Window(Gtk.ApplicationWindow):
             if playlist_dialog.run() == Gtk.ResponseType.ACCEPT:
                 playlists.add_to_playlist(
                     playlist_dialog.get_selected(), selected_songs)
-            self.toolbar.set_selection_mode(False)
+            self.headerbar.props.selection_mode = False
             playlist_dialog.destroy()
 
         self._stack.get_visible_child().get_selected_songs(callback)
+
+    @log
+    def set_player_visible(self, visible):
+        """Set PlayWidget action visibility
+
+        :param bool visible: actionbar visibility
+        """
+        self.player_toolbar.set_visible(visible)
